# Generated by roxygen2: do not edit by hand

export(ZL)
export(add_oth_pvals)
export(calc_gam_star)
export(find_mag)
export(gen_boot_sample)
export(ic.data.examp)
export(ic.pearson)
export(ic.proj.rr)
export(ic.proj.rr.nolas)
export(l_p_norm)
export(look_IC)
export(mag_for_pow)
export(make_data)
export(mv_pn_test)
export(pval_for_mag)
export(rr.msm.ic)
export(rr.msm.jn.ic)
export(test.control)
importFrom(dplyr,group_by)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(rlang,.data)
importFrom(tidyr,pivot_longer)
