AMP: Automatic Mathematical Parser
==================================

Python 2.7 library that parses and evaluates any string numerical mathematical expression, with support for complex numbers and mathematical constants, as well as scientific notation and a wide range of mathematical functions.

Documentation
-------------
The documentation available as of the date of this release is included in the docs/ directory.

Installation
------------
Installation of this package uses pip, included by default in Python 2.7.9 and later. If your computer doesn’t have pip, download `this file
<http://bootstrap.pypa.io/get-pip.py>`_. Then open up your terminal and enter the command shown below::

 python get-pip.py

Installation of the AMP package can be done by opening your computer's terminal and entering the following::

 pip install AMP

License
-------
See the file called LICENSE.txt.

Links
-----
The most up-to-date version of this software can be found at:

* https://github.com/ioguntol/Automatic-Mathematical-Parser
* https://pypi.python.org/pypi/AMP/

Contact
-------
The developer, Ini Oguntola, can be contacted at ioguntol@gmail.com.
