################################################################
#### Estimating Influence Curve and parameter for median
#### Adam Elder
#### This script will contain all of the various proceedures
#### used for estimating both the parameter, and the influence
#### curves used for estimating the median.
#### These functions will be given an entire dataset, and will
#### return the estimate for the parameter, and for each
#### estimated influence curve at each observation.
################################################################

#### The following function will take a set of observations, and return the
#### estimated covariance matrix for the estimates of the spearman correlation.
#### Estimates of the covariance are generated using the empirical influence
#### function.  The first column of your data should correspond to the
#### variable of interest (the variable for which spearman correlation is
#### calculated.

ic.median <- function(...) {
  stop("This function has not yet been written")
}
